import {
  ConstructorType,
  DecoratorType,
  MetadataKeyType,
  Metadata,
  TypeUtil,
} from '@overflow/commons/core/type';

import * as Constants from './constants';
import {
  Annotation,
} from './annotation';

export class Decorator {
  public static create = (AnnotationClass: ConstructorType<Annotation>) => {

    return (...handlerArgs: any[]) => {
      let annotation: Annotation = new AnnotationClass(...handlerArgs);
      let name: string = AnnotationClass.name;

      return (...decoratorArgs: any[]) => {
        let decoratorType: DecoratorType = TypeUtil.getDecoratorType(decoratorArgs);

        const [target, propertyKey, descriptorOrParameterIndex] = decoratorArgs;

        // let type = typeof decoratorArgs[0] === 'function' ? decoratorArgs[0].prototype : decoratorArgs[0];
        // let clazz = TypeUtil.getClass(decoratorArgs[0]);
        if (typeof(annotation.metadataKey) === 'undefined') {
          throw new Error(`Annotation[@${name}] must be implements metadataKey().`);
        }
        let metadataKey: MetadataKeyType = annotation.metadataKey.call(annotation);
        Metadata.set(metadataKey, annotation, target, propertyKey);

        switch(decoratorType) {
        case DecoratorType.CLASS:
          if (typeof(annotation.classDecorator) === 'undefined') {
            throw new Error(`Cannot apply @${name} decorator on Class.`);
          }

          return annotation.classDecorator.call(annotation, target);
        case DecoratorType.PROPERTY:
          if (typeof(annotation.propertyDecorator) === 'undefined') {
            throw new Error(`Cannot apply @${name} decorator on Property.`);
          }

          return annotation.propertyDecorator.call(annotation, target, propertyKey);
        case DecoratorType.METHOD:
          if (typeof(annotation.methodDecorator) === 'undefined') {
            throw new Error(`Cannot apply @${name} decorator on Method.`);
          }

          return annotation.methodDecorator.call(annotation, target, propertyKey, descriptorOrParameterIndex);
        case DecoratorType.PARAMETER:
          if (typeof(annotation.parameterDecorator) === 'undefined') {
            throw new Error(`Cannot apply @${name} decorator on Parameter.`);
          }

          return annotation.parameterDecorator.call(annotation, target, propertyKey, descriptorOrParameterIndex);
        default:
          throw new Error(`Cannot determine decorator[@${name}] type.`);
        }
      };
    };
  }

}
