import {
  MetadataKeyType,
  PropertyKeyType,
  TypeUtil,
} from '@overflow/commons/core/type';

import {
  Annotation,
  Decorator,
} from '@overflow/commons/decorator';

export const ActionMappingMetadataKey = Symbol('ActionMappingAnnotation');

export interface ActionMappingAnnotationAttributes {
  value: string;
}

export class ActionMappingAnnotation extends Annotation {
  public readonly attributes: ActionMappingAnnotationAttributes;

  public constructor(value: string | ActionMappingAnnotationAttributes) {
    super();

    if (undefined === value) {
      throw new Error(`value attribute must be specified.`);
    }

    if (TypeUtil.isString(value)) {
      this.attributes = {
        value: <string>value,
      };
    } else {
      this.attributes = <ActionMappingAnnotationAttributes>value;
    }
  }

  public metadataKey(): MetadataKeyType {
    return ActionMappingMetadataKey;
  }

  public classDecorator<TFunction extends Function>(target: TFunction): TFunction | void {
    console.log('ActionMapping');
  }

  public methodDecorator<T>(target: Object, propertyKey: PropertyKeyType,
                            descriptor: TypedPropertyDescriptor<T>): TypedPropertyDescriptor<T> | void {
    console.log('ActionMapping');
  }

}

export const ActionMapping = Decorator.create(ActionMappingAnnotation);
