import {
  MetadataKeyType,
  TypeUtil,
} from '@overflow/commons/core/type';

import {
  Annotation,
  Decorator,
} from '@overflow/commons/decorator';

export const ReducerMetadataKey = Symbol('ReducerAnnotation');

export interface ReducerAnnotationAttributes {
  name?: string;
}

export class ReducerAnnotation extends Annotation {
  public readonly attributes: ReducerAnnotationAttributes;

  public constructor(name: string | ReducerAnnotationAttributes) {
    super();

    if (undefined === name) {
      this.attributes = {
        name: '',
      };
      return;
    }

    if (TypeUtil.isString(name)) {
      this.attributes = {
        name: <string>name,
      };
    } else {
      this.attributes = <ReducerAnnotationAttributes>name;
    }
  }

  public metadataKey(): MetadataKeyType {
    return ReducerMetadataKey;
  }

  public classDecorator<TFunction extends Function>(target: TFunction): TFunction | void {
    console.log('Reducer');
  }
}

export const Reducer = Decorator.create(ReducerAnnotation);
