import {
  MetadataKeyType,
  PropertyKeyType,
  TypeUtil,
} from '@overflow/commons/core/type';

import {
  Annotation,
  Decorator,
} from '@overflow/commons/decorator';

export const RestAPIMetadataKey = Symbol('RestAPIAnnotation');

export interface RestAPIAnnotationAttributes {
  entryPath: string;
}

export class RestAPIAnnotation extends Annotation {
  public readonly attributes: RestAPIAnnotationAttributes;

  public constructor(entryPath: string | RestAPIAnnotationAttributes) {
    super();

    if (undefined === entryPath) {
      throw new Error(`entryPath attribute must be specified.`);
    }

    if (TypeUtil.isString(entryPath)) {
      this.attributes = {
        entryPath: <string>entryPath,
      };
    } else {
      this.attributes = <RestAPIAnnotationAttributes>entryPath;
    }
  }

  public metadataKey(): MetadataKeyType {
    return RestAPIMetadataKey;
  }

  public methodDecorator<T>(target: Object, propertyKey: PropertyKeyType,
                            descriptor: TypedPropertyDescriptor<T>): TypedPropertyDescriptor<T> | void {
    console.log('RestAPI');
  }
}

export const RestAPI = Decorator.create(RestAPIAnnotation);
