import {
  MetadataKeyType,
  PropertyKeyType,
  TypeUtil,
} from '@overflow/commons/core/type';

import {
  Annotation,
  Decorator,
} from '@overflow/commons/decorator';

export const RpcAPIMetadataKey = Symbol('RpcAPIAnnotation');

export interface RpcAPIAnnotationAttributes {
  method: string;
}

export class RpcAPIAnnotation extends Annotation {
  public readonly attributes: RpcAPIAnnotationAttributes;

  public constructor(method: string | RpcAPIAnnotationAttributes) {
    super();

    if (undefined === method) {
      throw new Error(`method attribute must be specified.`);
    }

    if (TypeUtil.isString(method)) {
      this.attributes = {
        method: <string>method,
      };
    } else {
      this.attributes = <RpcAPIAnnotationAttributes>method;
    }
  }

  public metadataKey(): MetadataKeyType {
    return RpcAPIMetadataKey;
  }

  public methodDecorator<T>(target: Object, propertyKey: PropertyKeyType,
                            descriptor: TypedPropertyDescriptor<T>): TypedPropertyDescriptor<T> | void {
    console.log('RestAPI');
  }
}

export const RpcAPI = Decorator.create(RpcAPIAnnotation);
